/*
 *========================================================================
 * $Id: get_proc_meminfo.c,v 1.4 2002/03/08 05:13:43 rgb Exp $
 *
 * See copyright in copyright.h and the accompanying file COPYING
 *========================================================================
 */

#include "xmlsysd.h"

/*
 * This routine parses /proc/stat and packs its values into doc.
 */

void get_proc_meminfo(xmlNodePtr proc)
{

 int i,numfields;
 struct timeval tv;
 xmlNodePtr meminfo,memory,swap,meminfovalue;

 if(verbose==102){
   fprintf(stderr,"Starting get_proc_meminfo().\n");
 }

 /* 
  * Create a timestamped child of meminfo node.  Everything below
  * will belong to this
  */
 meminfo = xmlNewChild(proc,NULL,"meminfo",NULL);
 gettimeofday(&tv,0);
 sprintf(outbuf,"%d",tv.tv_sec);
 xmlSetProp(meminfo,"tv_sec",outbuf);
 sprintf(outbuf,"%d",tv.tv_usec);
 xmlSetProp(meminfo,"tv_usec",outbuf);


 /* 
  * Now, for a clever trick.  We RESET the files without actually
  * closing or reopening them.  This should save the overhead of
  * an open/close (presumed relatively large, as one has to 
  * allocate/free certain kernel structures and meminfo the file in question 
  * on EACH open/close).
  */

 /* PROC_MEMINFO */
 errno = 2;
 if(stat_fd[PROC_MEMINFO]){
   rewind(stat_fd[PROC_MEMINFO]);	/* void, so tough to check errors */
 } else {
   xmlSetProp(meminfo,"available","no");
   return;
 }
 if(errno == EBADF){
   fprintf(stderr,"Error: The %s file descriptor/stream is not seekable.\n",procpaths[PROC_MEMINFO]);
   fclose(stat_fd[PROC_MEMINFO]); 
   fprintf(stderr,"Closing and reopening %s.\n",procpaths[PROC_MEMINFO]);
   stat_fd[PROC_MEMINFO] = fopen(procpaths[PROC_MEMINFO],"r");
 }

 if(verbose == 102){
   fprintf(stderr,"Rewound %s.\n",procpaths[PROC_MEMINFO]);
 }

 while(TRUE){

   /* Normal EOF causes break from while loop */
   if((fgets(statbuf,K,stat_fd[PROC_MEMINFO]) == NULL)) break;

   if(verbose == 102){
     fprintf(stderr,"Parsing %s\n",statbuf);
   }

   /* parse the line into fields */
   numfields = parse(statbuf,fields,MAXFIELDNUMBER,K);

   /*
    * Now we go down a simple lookup table to assemble each statistic
    * by name.  fields[] now contains the parsed fields from a line of
    * /proc/meminfo.  We identify the line by its first entry (the name
    * of the /proc/meminfo entity) and use the following values to build 
    * one or more statistics.
    */

   /* Get all "Mem: values from a single line.  No conversion. */
   if(strncmp(fields[0],"Mem",3) == 0 && strlen(fields[0]) == 3){
     memory = xmlNewChild(meminfo,NULL,"memory",NULL);
     meminfovalue = xmlNewChild(memory,NULL,"total",fields[1]);
     meminfovalue = xmlNewChild(memory,NULL,"used",fields[2]);
     meminfovalue = xmlNewChild(memory,NULL,"free",fields[3]);
     meminfovalue = xmlNewChild(memory,NULL,"shared",fields[4]);
     meminfovalue = xmlNewChild(memory,NULL,"buffers",fields[5]);
     meminfovalue = xmlNewChild(memory,NULL,"cached",fields[6]);
   }

   /* Get all "Mem: values from a single line.  No conversion. */
   if(strncmp(fields[0],"Swap",4) == 0 && strlen(fields[0]) == 4){
     swap = xmlNewChild(meminfo,NULL,"swap",NULL);
     meminfovalue = xmlNewChild(swap,NULL,"total",fields[1]);
     meminfovalue = xmlNewChild(swap,NULL,"used",fields[2]);
     meminfovalue = xmlNewChild(swap,NULL,"free",fields[3]);
   }
 }

}

